@@ -1,18 +1,47 @@
 <?php
+/**
+ * Zend Framework
+ *
+ * LICENSE
+ *
+ * This source file is subject to the new BSD license that is bundled
+ * with this package in the file LICENSE.txt.
+ * It is also available through the world-wide-web at this URL:
+ * http://framework.zend.com/license/new-bsd
+ * If you did not receive a copy of the license and are unable to
+ * obtain it through the world-wide-web, please send an email
+ * to license@zend.com so we can send you a copy immediately.
+ *
+ * @category   Zend
+ * @package    Zend_Controller
+ * @subpackage UnitTests
+ * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
+ * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
+ */
+
 // Call Zend_Controller_Dispatcher_StandardTest::main() if this source file is executed directly.
 if (!defined("PHPUnit_MAIN_METHOD")) {
     define("PHPUnit_MAIN_METHOD", "Zend_Controller_Dispatcher_StandardTest::main");
 }
 
-require_once dirname(__FILE__) . '/../../../TestHelper.php';
-
 require_once 'Zend/Controller/Dispatcher/Standard.php';
 require_once 'Zend/Controller/Action/HelperBroker.php';
+require_once 'Zend/Controller/Front.php';
 require_once 'Zend/Controller/Request/Http.php';
 require_once 'Zend/Controller/Request/Simple.php';
 require_once 'Zend/Controller/Response/Cli.php';
 
-class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase 
+/**
+ * @category   Zend
+ * @package    Zend_Controller
+ * @subpackage UnitTests
+ * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
+ * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @group      Zend_Controller
+ * @group      Zend_Controller_Dispatcher
+ */
+class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
 {
     protected $_dispatcher;
 
@@ -48,6 +77,15 @@ class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
         unset($this->_dispatcher);
     }
 
+    /**
+     * @group ZF-9800
+     */
+    public function testFormatModuleName()
+    {
+        $this->assertEquals('Test', $this->_dispatcher->formatModuleName('test'));
+        $this->assertEquals('TestFoo', $this->_dispatcher->formatModuleName('test-foo'));
+    }
+
     public function testFormatControllerName()
     {
         $this->assertEquals('IndexController', $this->_dispatcher->formatControllerName('index'));
@@ -105,6 +143,30 @@ class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
         $this->assertFalse($this->_dispatcher->isDispatchable($request), var_export($this->_dispatcher->getControllerDirectory(), 1));
     }
 
+    /**
+     * @group ZF-8222
+     */
+    public function testIsDispatchableManuallyIncludedController()
+    {
+        require_once dirname(__FILE__) . '/../_files/ManuallyIncludedControllers.php';
+        $request = new Zend_Controller_Request_Http();
+
+
+        $this->_dispatcher->setParam('prefixDefaultModule', true);
+
+        $request->setControllerName('included');
+        $this->assertFalse($this->_dispatcher->isDispatchable($request));
+
+        $request->setControllerName('included-prefix');
+        $this->assertTrue($this->_dispatcher->isDispatchable($request));
+
+        $this->_dispatcher->setParam('prefixDefaultModule', false);
+
+        $request->setModuleName('admin');
+        $request->setControllerName('included-admin');
+        $this->assertTrue($this->_dispatcher->isDispatchable($request));
+    }
+
     public function testSetGetResponse()
     {
         $response = new Zend_Controller_Response_Cli();
@@ -195,6 +257,29 @@ class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
         }
     }
 
+    /**
+     * @group ZF-3465
+     */
+    public function testUsingDefaultControllerAlwaysShouldRewriteActionNameToDefault()
+    {
+        $request = new Zend_Controller_Request_Http();
+        $request->setControllerName('bogus');
+        $request->setActionName('really');
+        $request->setParam('action', 'really'); // router sets action as a param
+        $response = new Zend_Controller_Response_Cli();
+
+        $this->_dispatcher->setParam('useDefaultControllerAlways', true);
+
+        try {
+            $this->_dispatcher->dispatch($request, $response);
+        } catch (Zend_Controller_Dispatcher_Exception $e) {
+            $this->fail('Exception should not be raised when useDefaultControllerAlways set; message: ' . $e->getMessage());
+        }
+
+        $this->assertEquals('index', $request->getControllerName());
+        $this->assertEquals('index', $request->getActionName());
+    }
+
     public function testDispatchInvalidControllerUsingDefaultsWithDefaultModule()
     {
         $request = new Zend_Controller_Request_Http();
@@ -395,7 +480,7 @@ class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
     }
 
     /**
-     * @see ZF-3034
+     * @group ZF-3034
      */
     public function testIsValidModuleShouldNormalizeModuleName()
     {
@@ -459,6 +544,42 @@ class Zend_Controller_Dispatcher_StandardTest extends PHPUnit_Framework_TestCase
         $this->assertTrue(class_exists($test));
     }
 
+    /**
+     * @group ZF-9800
+     */
+    public function testLoadClassLoadsControllerInSpecifiedModuleWithHyphenatedModuleName()
+    {
+        $front = Zend_Controller_Front::getInstance();
+        $front->addModuleDirectory(dirname(__FILE__) . '/../_files/modules');
+        $dispatcher = $front->getDispatcher();
+
+        $request = new Zend_Controller_Request_Simple();
+        $request->setControllerName('foo')
+                ->setModuleName('baz-bat');
+        $class = $dispatcher->getControllerClass($request);
+        $this->assertEquals('FooController', $class);
+        $test = $dispatcher->loadClass($class);
+        $this->assertEquals('BazBat_FooController', $test);
+        $this->assertTrue(class_exists($test));
+    }
+
+    /**
+     * @group ZF-9800
+     */
+    public function testDispatcherCanDispatchControllersFromModuleWithHyphenatedName()
+    {
+        $front = Zend_Controller_Front::getInstance();
+        $front->addModuleDirectory(dirname(__FILE__) . '/../_files/modules');
+        $dispatcher = $front->getDispatcher();
+
+        $request = new Zend_Controller_Request_Simple();
+        $request->setModuleName('baz-bat')->setControllerName('foo');
+        $response = new Zend_Controller_Response_Cli();
+        $dispatcher->dispatch($request, $response);
+        $body = $dispatcher->getResponse()->getBody();
+        $this->assertContains("BazBat_FooController::indexAction() called", $body, $body);
+    }
+
     public function testLoadClassLoadsControllerInDefaultModuleWithModulePrefixWhenRequested()
     {
         Zend_Controller_Front::getInstance()
