@@ -1,10 +1,29 @@
 <?php
 /**
+ * Zend Framework
+ *
+ * LICENSE
+ *
+ * This source file is subject to the new BSD license that is bundled
+ * with this package in the file LICENSE.txt.
+ * It is also available through the world-wide-web at this URL:
+ * http://framework.zend.com/license/new-bsd
+ * If you did not receive a copy of the license and are unable to
+ * obtain it through the world-wide-web, please send an email
+ * to license@zend.com so we can send you a copy immediately.
+ *
  * @category   Zend
  * @package    Zend_Controller
  * @subpackage UnitTests
+ * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
+ * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
+if (!defined('PHPUnit_MAIN_METHOD')) {
+    define('PHPUnit_MAIN_METHOD', 'Zend_Controller_Router_RewriteTest::main');
+}
+
 /** Zend_Controller_Router_Rewrite */
 require_once 'Zend/Controller/Router/Rewrite.php';
 
@@ -26,20 +45,36 @@ require_once 'Zend/Controller/Router/Route/Chain.php';
 /** Zend_Controller_Router_Route_Hostname */
 require_once 'Zend/Controller/Router/Route/Hostname.php';
 
-/** PHPUnit test case */
-require_once 'PHPUnit/Framework/TestCase.php';
+/** Zend_Uri_Http */
+require_once 'Zend/Uri/Http.php';
 
-require_once 'PHPUnit/Runner/Version.php';
 
 /**
  * @category   Zend
  * @package    Zend_Controller
  * @subpackage UnitTests
+ * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
+ * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @group      Zend_Controller
+ * @group      Zend_Controller_Router
  */
 class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 {
     protected $_router;
 
+    /**
+     * Runs the test methods of this class.
+     *
+     * @access public
+     * @static
+     */
+    public static function main()
+    {
+
+        $suite  = new PHPUnit_Framework_TestSuite("Zend_Controller_Router_RewriteTest");
+        $result = PHPUnit_TextUI_TestRunner::run($suite);
+    }
+
     public function setUp() {
         $this->_router = new Zend_Controller_Router_Rewrite();
         $front = Zend_Controller_Front::getInstance();
@@ -59,13 +94,13 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $routes = $this->_router->getRoutes();
 
         $this->assertEquals(1, count($routes));
-        $this->assertType('Zend_Controller_Router_Route', $routes['archive']);
+        $this->assertTrue($routes['archive'] instanceof Zend_Controller_Router_Route);
 
         $this->_router->addRoute('register', new Zend_Controller_Router_Route('register/:action', array('controller' => 'profile', 'action' => 'register')));
         $routes = $this->_router->getRoutes();
 
         $this->assertEquals(2, count($routes));
-        $this->assertType('Zend_Controller_Router_Route', $routes['register']);
+        $this->assertTrue($routes['register'] instanceof Zend_Controller_Router_Route);
     }
 
     public function testAddRoutes()
@@ -79,8 +114,8 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $values = $this->_router->getRoutes();
 
         $this->assertEquals(2, count($values));
-        $this->assertType('Zend_Controller_Router_Route', $values['archive']);
-        $this->assertType('Zend_Controller_Router_Route', $values['register']);
+        $this->assertTrue($values['archive'] instanceof Zend_Controller_Router_Route);
+        $this->assertTrue($values['register'] instanceof Zend_Controller_Router_Route);
     }
 
     public function testHasRoute()
@@ -98,7 +133,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $route = $this->_router->getRoute('archive');
 
-        $this->assertType('Zend_Controller_Router_Route', $route);
+        $this->assertTrue($route instanceof Zend_Controller_Router_Route);
         $this->assertSame($route, $archive);
     }
 
@@ -116,7 +151,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         try {
             $route = $this->_router->removeRoute('archive');
         } catch (Zend_Controller_Router_Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
             return true;
         }
 
@@ -128,7 +163,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         try {
             $route = $this->_router->getRoute('bogus');
         } catch (Zend_Controller_Router_Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
             return true;
         }
 
@@ -141,7 +176,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $token = $this->_router->route($request);
 
-        $this->assertType('Zend_Controller_Request_Http', $token);
+        $this->assertTrue($token instanceof Zend_Controller_Request_Http);
     }
 
     public function testRouteWithIncorrectRequest()
@@ -152,7 +187,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
             $token = $this->_router->route($request);
             $this->fail('Should throw an Exception');
         } catch (Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
         }
     }
 
@@ -163,7 +198,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $token = $this->_router->route($request);
 
         $routes = $this->_router->getRoutes();
-        $this->assertType('Zend_Controller_Router_Route_Module', $routes['default']);
+        $this->assertTrue($routes['default'] instanceof Zend_Controller_Router_Route_Module);
     }
 
     public function testDefaultRouteWithEmptyAction()
@@ -221,10 +256,15 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->_router->addRoute('default', new Zend_Controller_Router_Route(':controller/:action'));
 
-        $token = $this->_router->route($request);
+        try {
+            $token = $this->_router->route($request);
+            $this->fail('An expected Zend_Controller_Router_Exception was not raised');
+        } catch (Zend_Controller_Router_Exception $expected) {
+            $this->assertEquals('No route matched the request', $expected->getMessage());
+        }
 
-        $this->assertNull($token->getControllerName());
-        $this->assertNull($token->getActionName());
+        $this->assertNull($request->getControllerName());
+        $this->assertNull($request->getActionName());
     }
 
     public function testDefaultRouteMatched()
@@ -268,14 +308,14 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
             $route = $this->_router->getCurrentRoute();
             $this->fail();
         } catch (Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
         }
 
         try {
             $route = $this->_router->getCurrentRouteName();
             $this->fail();
         } catch (Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
         }
 
         $token = $this->_router->route($request);
@@ -288,7 +328,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         }
 
         $this->assertEquals('default', $name);
-        $this->assertType('Zend_Controller_Router_Route_Module', $route);
+        $this->assertTrue($route instanceof Zend_Controller_Router_Route_Module);
     }
 
     public function testAddConfig()
@@ -299,13 +339,13 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->_router->addConfig($config, 'routes');
 
-        $this->assertType('Zend_Controller_Router_Route_Static', $this->_router->getRoute('news'));
-        $this->assertType('Zend_Controller_Router_Route', $this->_router->getRoute('archive'));
+        $this->assertTrue($this->_router->getRoute('news') instanceof Zend_Controller_Router_Route_Static);
+        $this->assertTrue($this->_router->getRoute('archive') instanceof Zend_Controller_Router_Route);
 
         try {
             $this->_router->addConfig($config, 'database');
         } catch (Exception $e) {
-            $this->assertType('Zend_Controller_Router_Exception', $e);
+            $this->assertTrue($e instanceof Zend_Controller_Router_Exception);
             return true;
         }
     }
@@ -318,8 +358,8 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->_router->addConfig($config->routes);
 
-        $this->assertType('Zend_Controller_Router_Route_Static', $this->_router->getRoute('news'));
-        $this->assertType('Zend_Controller_Router_Route', $this->_router->getRoute('archive'));
+        $this->assertTrue($this->_router->getRoute('news') instanceof Zend_Controller_Router_Route_Static);
+        $this->assertTrue($this->_router->getRoute('archive') instanceof Zend_Controller_Router_Route);
     }
 
     public function testAddConfigWithRootNode()
@@ -330,8 +370,8 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->_router->addConfig($config);
 
-        $this->assertType('Zend_Controller_Router_Route_Static', $this->_router->getRoute('news'));
-        $this->assertType('Zend_Controller_Router_Route', $this->_router->getRoute('archive'));
+        $this->assertTrue($this->_router->getRoute('news') instanceof Zend_Controller_Router_Route_Static);
+        $this->assertTrue($this->_router->getRoute('archive') instanceof Zend_Controller_Router_Route);
     }
 
     public function testRemoveDefaultRoutes()
@@ -339,7 +379,12 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $request = new Zend_Controller_Router_RewriteTest_Request('http://localhost/ctrl/act');
         $this->_router->removeDefaultRoutes();
 
-        $token = $this->_router->route($request);
+        try {
+            $token = $this->_router->route($request);
+            $this->fail('An expected Zend_Controller_Router_Exception was not raised');
+        } catch (Zend_Controller_Router_Exception $expected) {
+            $this->assertEquals('No route matched the request', $expected->getMessage());
+        }
 
         $routes = $this->_router->getRoutes();
         $this->assertEquals(0, count($routes));
@@ -442,8 +487,8 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $this->_router->addRoute('req', new Zend_Controller_Router_Route_Interface_Mockup());
         $routeRequest = $this->_router->getRoute('req')->getRequest();
 
-        $this->assertType('Zend_Controller_Request_Abstract', $request);
-        $this->assertType('Zend_Controller_Request_Abstract', $routeRequest);
+        $this->assertTrue($request instanceof Zend_Controller_Request_Abstract);
+        $this->assertTrue($routeRequest instanceof Zend_Controller_Request_Abstract);
         $this->assertSame($request, $routeRequest);
     }
 
@@ -451,7 +496,7 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
     {
         $request = new Zend_Controller_Router_RewriteTest_Request('http://localhost/en/bar');
         $request->setParam('path', 'v2test');
-        
+
         $route = new Zend_Controller_RouterTest_RouteV2_Stub('not-important');
         $this->_router->addRoute('foo', $route);
 
@@ -459,11 +504,9 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->assertEquals('v2test', $token->getParam('path'));
     }
-        
+
     public function testRoutingChainedRoutes()
     {
-        $this->markTestSkipped('Route features not ready');
-        
         $request = new Zend_Controller_Router_RewriteTest_Request('http://localhost/foo/bar');
 
         $foo = new Zend_Controller_Router_Route('foo', array('foo' => true));
@@ -481,98 +524,90 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(true, $token->getParam('foo'));
         $this->assertEquals(true, $token->getParam('bar'));
     }
-    
+
     public function testRouteWithHostnameChain()
     {
         $request = new Zend_Controller_Router_RewriteTest_Request('http://www.zend.com/bar');
-        
+
         $foo = new Zend_Controller_Router_Route_Hostname('nope.zend.com', array('module' => 'nope-bla', 'bogus' => 'bogus'));
         $bar = new Zend_Controller_Router_Route_Hostname('www.zend.com', array('module' => 'www-bla'));
-        
+
         $bla = new Zend_Controller_Router_Route_Static('bar', array('controller' => 'foo', 'action' => 'bar'));
-        
+
         $chainMatch = new Zend_Controller_Router_Route_Chain();
         $chainMatch->chain($bar)->chain($bla);
-        
+
         $chainNoMatch = new Zend_Controller_Router_Route_Chain();
         $chainNoMatch->chain($foo)->chain($bla);
-               
+
         $this->_router->addRoute('match',    $chainMatch);
         $this->_router->addRoute('no-match', $chainNoMatch);
-        
+
         $token = $this->_router->route($request);
 
         $this->assertEquals('www-bla', $token->getModuleName());
         $this->assertEquals('foo', $token->getControllerName());
         $this->assertEquals('bar', $token->getActionName());
         $this->assertNull($token->getParam('bogus'));
-    } 
-    
+    }
+
     public function testAssemblingWithHostnameHttp()
     {
-        $this->markTestSkipped('Router features not ready');
-        
         $route = new Zend_Controller_Router_Route_Hostname('www.zend.com');
 
         $this->_router->addRoute('hostname-route', $route);
 
         $this->assertEquals('http://www.zend.com', $this->_router->assemble(array(), 'hostname-route'));
     }
-    
+
     public function testAssemblingWithHostnameHttps()
     {
-        $this->markTestSkipped('Router features not ready');
-        
         $backupServer = $_SERVER;
         $_SERVER['HTTPS'] = 'on';
-        
+
         $route = new Zend_Controller_Router_Route_Hostname('www.zend.com');
 
         $this->_router->addRoute('hostname-route', $route);
 
         $this->assertEquals('https://www.zend.com', $this->_router->assemble(array(), 'hostname-route'));
-        
+
         $_SERVER = $backupServer;
     }
-    
+
     public function testAssemblingWithHostnameThroughChainHttp()
     {
-        $this->markTestSkipped('Router features not ready');
-        
         $foo = new Zend_Controller_Router_Route_Hostname('www.zend.com');
         $bar = new Zend_Controller_Router_Route_Static('bar');
 
         $chain = new Zend_Controller_Router_Route_Chain();
         $chain->chain($foo)->chain($bar);
-        
+
         $this->_router->addRoute('foo-bar', $chain);
 
         $this->assertEquals('http://www.zend.com/bar', $this->_router->assemble(array(), 'foo-bar'));
     }
-    
+
     public function testAssemblingWithHostnameWithChainHttp()
     {
-        $this->markTestSkipped('Router features not ready');
-        
         $foo = new Zend_Controller_Router_Route_Hostname('www.zend.com');
         $bar = new Zend_Controller_Router_Route_Static('bar');
 
         $chain = $foo->chain($bar);
-        
+
         $this->_router->addRoute('foo-bar', $chain);
 
         $this->assertEquals('http://www.zend.com/bar', $this->_router->assemble(array(), 'foo-bar'));
     }
-    
+
     public function testAssemblingWithNonFirstHostname()
     {
         $this->markTestSkipped('Router features not ready');
-        
+
         $foo = new Zend_Controller_Router_Route_Static('bar');
         $bar = new Zend_Controller_Router_Route_Hostname('www.zend.com');
 
         $foo->chain($bar);
-        
+
         $this->_router->addRoute('foo-bar', $foo);
 
         $this->assertEquals('bar/www.zend.com', $this->_router->assemble(array(), 'foo-bar'));
@@ -584,12 +619,12 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
     public function testRouteShouldMatchEvenWithTrailingSlash()
     {
         $route = new Zend_Controller_Router_Route(
-            'blog/articles/:id', 
+            'blog/articles/:id',
             array(
                 'controller' => 'blog',
                 'action'     => 'articles',
                 'id'         => 0,
-            ), 
+            ),
             array(
                 'id' => '[0-9]+',
             )
@@ -603,6 +638,133 @@ class Zend_Controller_Router_RewriteTest extends PHPUnit_Framework_TestCase
 
         $this->assertEquals('2006', $token->getParam('id', false));
     }
+
+    public function testGlobalParam()
+    {
+        $route = new Zend_Controller_Router_Route(
+            ':lang/articles/:id',
+            array(
+                'controller' => 'blog',
+                'action'     => 'articles',
+                'id'         => 0,
+            )
+        );
+        $this->_router->addRoute('article-id', $route);
+        $this->_router->setGlobalParam('lang', 'de');
+
+        $url = $this->_router->assemble(array('id' => 1), 'article-id');
+
+        $this->assertEquals('/de/articles/1', $url);
+    }
+
+    public function testGlobalParamOverride()
+    {
+        $route = new Zend_Controller_Router_Route(
+            ':lang/articles/:id',
+            array(
+                'controller' => 'blog',
+                'action'     => 'articles',
+                'id'         => 0,
+            )
+        );
+        $this->_router->addRoute('article-id', $route);
+        $this->_router->setGlobalParam('lang', 'de');
+
+        $url = $this->_router->assemble(array('id' => 1, 'lang' => 'en'), 'article-id');
+
+        $this->assertEquals('/en/articles/1', $url);
+    }
+
+    public function testChainNameSeparatorIsSetCorrectly() {
+        $separators = array('_','unitTestSeparator','-');
+        $results = array();
+
+        foreach($separators as $separator) {
+            $this->_router->setChainNameSeparator($separator);
+            $results[] = $this->_router->getChainNameSeparator();
+        }
+
+        $this->assertEquals($separators, $results);
+    }
+
+    public function testChainNameSeparatorisUsedCorrectly() {
+        $config = new Zend_Config(array('chains' => array(
+            'type'=>'Zend_Controller_Router_Route_Static',
+            'route'=>'foo',
+            'chains'=> array('bar'=>
+                array('type'=>'Zend_Controller_Router_Route_Static',
+                    'route'=>'bar',
+                    'defaults'=>array(
+                    'module'=>'module',
+                    'controller'=>'controller',
+                    'action'=>'action'))))));
+        $this->_router->setChainNameSeparator('_separator_')
+                      ->addConfig($config);
+        $url = $this->_router->assemble(array(),'chains_separator_bar');
+        $this->assertEquals('/foo/bar',$url);
+    }
+
+    public function testRequestParamsUsedAsGlobalParam()
+    {
+        $route = new Zend_Controller_Router_Route(
+            '/articles/:id',
+            array(
+                'controller' => 'blog',
+                'action'     => 'articles',
+            )
+        );
+
+        $request = Zend_Controller_Front::getInstance()->getRequest();
+        $request->setParam('id', 777);
+
+        $this->_router->addRoute('article-id', $route);
+        $this->_router->useRequestParametersAsGlobal(true);
+        $this->_router->route($request);
+
+        $url = $this->_router->assemble(array(), 'article-id');
+
+        $this->assertEquals('/articles/777', $url);
+    }
+
+
+    /**
+     * Test that it is possible to generate a URL with a numerical key
+     *
+     * @since  2010-06-11
+     * @group  ZF-8914
+     * @covers Zend_Controller_Router_Rewrite::assemble
+     */
+    public function testCanGenerateNumericKeyUri()
+    {
+        $this->_router->addRoute(
+            'default',
+            new Zend_Controller_Router_Route(
+                ':controller/:action/*',
+                array('controller' => 'index', 'action' => 'index')
+            )
+       );
+
+       $params = array(
+            'controller' => 'index',
+            'action'     => 'index',
+            '2'          => 'foo',
+            'page'       => 'bar',
+        );
+
+        $this->assertEquals(
+            '/index/index/2/foo/page/bar',
+            $this->_router->assemble($params)
+        );
+    }
+    
+    /**
+     * @group ZF-11393
+     * @expectedException Zend_Controller_Router_Exception
+     */
+    public function testCallingAssembleWithNullArgumentShouldThrowException()
+    {
+        $this->_router->assemble(null);
+    }
 }
 
 
@@ -615,7 +777,7 @@ class Zend_Controller_Router_RewriteTest_Request extends Zend_Controller_Request
 {
     protected $_host;
     protected $_port;
-    
+
     public function __construct($uri = null)
     {
         if (null === $uri) {
@@ -625,7 +787,7 @@ class Zend_Controller_Router_RewriteTest_Request extends Zend_Controller_Request
         $uri = Zend_Uri_Http::fromString($uri);
         $this->_host = $uri->getHost();
         $this->_port = $uri->getPort();
-        
+
         parent::__construct($uri);
     }
 
@@ -666,9 +828,8 @@ class Zend_Controller_Router_RewriteTest_Request_Incorrect extends Zend_Controll
  *
  * @uses Zend_Controller_Request_Abstract
  */
-class Zend_Controller_RouterTest_RouteV2_Stub implements Zend_Controller_Router_Route_Interface
+class Zend_Controller_RouterTest_RouteV2_Stub extends Zend_Controller_Router_Route_Abstract
 {
-    public function getVersion() { return 2; }
     public function match($request) {
         return array('path', $request->getParam('path'));
     }
@@ -706,3 +867,7 @@ class Zend_Controller_Router_Route_Interface_Mockup implements Zend_Controller_R
         return $this->_request;
     }
 }
+
+if (PHPUnit_MAIN_METHOD == "Zend_Controller_Router_RewriteTest::main") {
+    Zend_Controller_Router_RewriteTest::main();
+}
